///File : F2fVerbs.cpp
//------------------------------------------------------------------------------
/*
    This file is part of forbidden2forget.

    Forbidden2forget is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Forbidden2forget is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with forbidden2forget.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "ticpp/ticpp.h"
#include "F2fVerbs.hpp"
//------------------------------------------------------------------------------
F2fVerbs::F2fVerbs()
{
    m_isOk = false;
    m_v.isCorrecting = true;
    m_v.idV = 0;

    m_v.fr.clear();
    m_v.bv.clear();
    m_v.pret.clear();
    m_v.pp.clear();

    m_v.nbVerbsAtBegining = 0;
    m_v.nbErrors = 0;
}
//------------------------------------------------------------------------------
bool F2fVerbs::isAbleToDeliverInfo() const
{
    if(!IsFinish() && m_v.idV != -1 && !IsProtected() && IsOk())
        return true;
    else
        return false;
}

//------------------------------------------------------------------------------
std::string F2fVerbs::GetCurrentFr() const
{
    if(isAbleToDeliverInfo())
        return m_v.fr[m_v.idV];
    else
        return "";
}
//------------------------------------------------------------------------------
std::string F2fVerbs::GetCurrentBV() const
{
    if(isAbleToDeliverInfo())
        return m_v.bv[m_v.idV];
    else
        return "";
}

//------------------------------------------------------------------------------
std::string F2fVerbs::GetCurrentPret() const
{
    if(isAbleToDeliverInfo())
        return m_v.pret[m_v.idV];
    else
        return "";
}

//------------------------------------------------------------------------------
std::string F2fVerbs::GetCurrentPP() const
{
    if(isAbleToDeliverInfo())
        return m_v.pp[m_v.idV];
    else
        return "";
}

//------------------------------------------------------------------------------
int F2fVerbs::GetCurrentNbLeft() const
{
    if(isAbleToDeliverInfo())
        return m_v.nbLeft[m_v.idV];
    else
        return 0;
}

//------------------------------------------------------------------------------
ticpp::Element *F2fVerbs::PrepareInfo(const std::string &nom, const std::string &info) const
{
    ticpp::Element *infoElement = new ticpp::Element(nom);
    infoElement->SetText(info);
    return infoElement;
}
//------------------------------------------------------------------------------

std::string F2fVerbs::intToString(int nb) const
{
    std::stringstream ss;
    std::string str;
    ss << nb;
    ss >> str;
    return str;
}

//------------------------------------------------------------------------------
int F2fVerbs::stringToInt(std::string nbStr) const
{
    std::stringstream ss;
    int nb;
    ss << nbStr;
    ss >> nb;
    return nb;
}
//------------------------------------------------------------------------------
ticpp::Element *F2fVerbs::PrepareVerb(const int i) const
{
    ticpp::Element *Verb = new ticpp::Element("verb");

    Verb->InsertEndChild(*PrepareInfo("fr", m_v.fr[i]));
    Verb->InsertEndChild(*PrepareInfo("bv", m_v.bv[i]));
    Verb->InsertEndChild(*PrepareInfo("pret", m_v.pret[i]));
    Verb->InsertEndChild(*PrepareInfo("pp", m_v.pp[i]));
    Verb->InsertEndChild(*PrepareInfo("nbLeft", intToString(m_v.nbLeft[i])));

    return Verb;
}

//------------------------------------------------------------------------------

void F2fVerbs::Save() const
{
    ticpp::Document myDoc(m_v.filename);

    ticpp::Declaration myDeclaration("1.0","","");

    ticpp::Element Version("version");
    Version.SetText("1");

    ticpp::Element Info("info");

    Info.InsertEndChild(*PrepareInfo("password", m_v.password));
    Info.InsertEndChild(*PrepareInfo("userName", m_v.userName));
    Info.InsertEndChild(*PrepareInfo("level", m_v.level));
    Info.InsertEndChild(*PrepareInfo("nbVerbsAtBegining", intToString(m_v.nbVerbsAtBegining)));
    Info.InsertEndChild(*PrepareInfo("nbErrors", intToString(m_v.nbErrors)));
    Info.InsertEndChild(*PrepareInfo("elapsedSeconds", intToString(m_v.elapsedSeconds)));

    ticpp::Element Verbs("verbs");
    for(int i = 0; i < m_v.fr.size(); i++)
    {
        Verbs.InsertEndChild(*PrepareVerb(i));
    }

    ticpp::Element f2fBase("forbidden2forget_file");
    f2fBase.InsertEndChild(Version);
    f2fBase.InsertEndChild(Info);
    f2fBase.InsertEndChild(Verbs);

    myDoc.InsertEndChild(myDeclaration);
    myDoc.InsertEndChild(f2fBase);

    myDoc.SaveFile();
}

//------------------------------------------------------------------------------

void F2fVerbs::SetPassword(std::string password)
{
    if(!IsProtected() && IsOk())
    {
        m_v.password = password;
    }
}

//------------------------------------------------------------------------------
void F2fVerbs::New(std::string originFilename, std::string newFilename, std::string userName)
{
    LoadFromFile(originFilename);
    m_v.filename = newFilename;
    m_v.userName = userName;

    Save();
}
//------------------------------------------------------------------------------
void F2fVerbs::AdToElapsedSeconds(int second)
{
    if(!IsProtected() && IsOk() && second >= 0)
    {
        m_v.elapsedSeconds += second;
    }
}

//------------------------------------------------------------------------------
void F2fVerbs::LoadFromFile(std::string filename)
{
    Clear();

    ticpp::Document myDoc(filename);
    myDoc.LoadFile();

    ticpp::Element f2fBase = *myDoc.FirstChild("forbidden2forget_file")->ToElement();
    if(f2fBase.FirstChild("version")->ToElement()->GetText() == "1")
    {
        // Infos de base :
        ticpp::Element Info = *f2fBase.FirstChild("info")->ToElement();

        m_v.password = Info.FirstChild("password")->ToElement()->GetTextOrDefault("");
        m_v.userName = Info.FirstChild("userName")->ToElement()->GetTextOrDefault("");
        m_v.level = Info.FirstChild("level")->ToElement()->GetTextOrDefault("");
        m_v.nbVerbsAtBegining = stringToInt(Info.FirstChild("nbVerbsAtBegining")->ToElement()->GetTextOrDefault(""));
        m_v.nbErrors = stringToInt(Info.FirstChild("nbErrors")->ToElement()->GetTextOrDefault(""));
        m_v.elapsedSeconds = stringToInt(Info.FirstChild("elapsedSeconds")->ToElement()->GetTextOrDefault(""));

        // Verbs :
        ticpp::Element Verbs = *f2fBase.FirstChild("verbs")->ToElement();

        ticpp::Element *VerbElement = Verbs.FirstChildElement("verb", false);
        while(VerbElement != NULL)
        {

            m_v.fr.push_back(VerbElement->FirstChild("fr")->ToElement()->GetText());
            m_v.bv.push_back(VerbElement->FirstChild("bv")->ToElement()->GetText());
            m_v.pret.push_back(VerbElement->FirstChild("pret")->ToElement()->GetText());
            m_v.pp.push_back(VerbElement->FirstChild("pp")->ToElement()->GetText());
            m_v.nbLeft.push_back(stringToInt(VerbElement->FirstChild("nbLeft")->ToElement()->GetText()));

            VerbElement = VerbElement->NextSiblingElement("verb", false);
        }
        m_v.filename = filename;

        m_isOk = true;
        m_v.isCorrecting = true;

        if(m_v.password == "")
        {
            m_v.isProtected = false;
        }
        else
        {
            m_v.isProtected = true;
        }

        m_v.idV = 0;
    }
    else
    {
        Clear();
        throw ticpp::Exception("Bad file version !");
    }
}

//------------------------------------------------------------------------------
std::string F2fVerbs::GetFilename() const
{
    return m_v.filename;
}

//------------------------------------------------------------------------------
void F2fVerbs::Clear()
{
    m_v.fr.clear();
    m_v.bv.clear();
    m_v.pret.clear();
    m_v.pp.clear();

    m_isOk = false;
    m_v.isCorrecting = true;
    m_v.idV = 0;

    m_v.filename = "";

    m_v.nbVerbsAtBegining = 0;
    m_v.nbErrors = 0;
}

//------------------------------------------------------------------------------
bool F2fVerbs::IsOk() const
{
    return m_isOk;
}

//------------------------------------------------------------------------------
bool F2fVerbs::IsPassword() const
{
    if(m_v.password != "")
        return true;
    else
        return false;
}

//------------------------------------------------------------------------------
bool F2fVerbs::IsCorrecting() const
{
    return m_v.isCorrecting;
}

//------------------------------------------------------------------------------
bool F2fVerbs::IsFinish() const
{
    return m_v.fr.empty();
}

//------------------------------------------------------------------------------
bool F2fVerbs::IsProtected() const
{
    return m_v.isProtected;
}

//------------------------------------------------------------------------------

void F2fVerbs::GetNewVerb()
{
    if(!IsFinish() && !IsProtected() && IsOk())
    {
        m_v.idV = hasard(0, m_v.fr.size() - 1);
        m_v.isCorrecting = false;
    }
}

//------------------------------------------------------------------------------
bool F2fVerbs::Correct(std::string bv, std::string pret, std::string pp)
{
    m_v.isCorrecting = true;
    if(isAbleToDeliverInfo())
    {
        if(bv == m_v.bv[m_v.idV] && pret == m_v.pret[m_v.idV] && pp == m_v.pp[m_v.idV])
        {
            if(m_v.nbLeft[m_v.idV] > 1)
            {
                m_v.nbLeft[m_v.idV]--;
            }
            else
            {
                m_v.fr.erase(m_v.fr.begin() + m_v.idV);
                m_v.bv.erase(m_v.bv.begin() + m_v.idV);
                m_v.pret.erase(m_v.pret.begin() + m_v.idV);
                m_v.pp.erase(m_v.pp.begin() + m_v.idV);
                m_v.nbLeft.erase(m_v.nbLeft.begin() + m_v.idV);

                m_v.idV = -1;
            }
            return true;
        }
        else
        {
            m_v.nbErrors++;
            return false;
        }
    }
    else
    {
        return false;
    }
}

//------------------------------------------------------------------------------
void F2fVerbs::TryPassword(std::string password)
{
    if(password == m_v.password)
        m_v.isProtected = false;
    else
        m_v.isProtected = true;

}

//------------------------------------------------------------------------------
std::string F2fVerbs::GetLevel() const
{
    if(!IsProtected() && IsOk())
    {
        return m_v.level;
    }
    else
    {
        return "";
    }
}

//------------------------------------------------------------------------------
std::string F2fVerbs::GetPupileName() const
{
    if(!IsProtected() && IsOk())
    {
        return m_v.userName;
    }
    else
    {
        return "";
    }
}

//------------------------------------------------------------------------------

int F2fVerbs::GetTotalVerbsAtBegining() const
{
    if(!IsProtected() && IsOk())
    {
        return m_v.nbVerbsAtBegining;
    }
    else
    {
        return 0;
    }
}

//------------------------------------------------------------------------------
int F2fVerbs::GetTotalVerbs() const
{
    if(!IsProtected() && IsOk())
    {
        return m_v.fr.size();
    }
    else
    {
        return 0;
    }
}

//------------------------------------------------------------------------------
int F2fVerbs::GetTotalLeft() const
{
    if(!IsProtected() && IsOk())
    {
        int total = 0;
        for(int i = 0; i < m_v.fr.size(); i++)
        {
            total += m_v.nbLeft[i];
        }
        return total;
    }
    else
    {
        return 0;
    }
}

//------------------------------------------------------------------------------
int F2fVerbs::GetWorkDuration() const
{
    if(!IsProtected() && IsOk())
    {
        return m_v.elapsedSeconds;
    }
    else
    {
        return 0;
    }
}

//------------------------------------------------------------------------------
int F2fVerbs::GetErrorNumber() const
{
    if(!IsProtected() && IsOk())
    {
        return m_v.nbErrors;
    }
    else
    {
        return 0;
    }
}

//------------------------------------------------------------------------------
int F2fVerbs::hasard(int min, int max) const
{
    return (int) (min + ((float) rand() / RAND_MAX * (max - min + 1)));
}
